//Project: WorldPrinter
//Homepage: www.HomoFaciens.de
//Author Norbert Heinz
//Version: 0.1
//Creation date: 16.01.2021
//This program is free software you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation version 3 of the License.
//This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
//For a copy of the GNU General Public License see http://www.gnu.org/licenses/
//
//compile with gcc commands-printer.c -o commands-printer -lm -lwiringPi -lwiringPiDev
//For details see: https://homofaciens.de/technics-machines-WorldPrinter_en.htm

#include <stdio.h>
#include <termios.h>
#include <sys/ioctl.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <sys/time.h>
#include <dirent.h>
#include <math.h>
#include <unistd.h>
#include <fcntl.h>
#include <lcd.h>

#include <wiringPi.h>


#define BAUDRATE B115200
//#define BAUDRATE B230400
//#define ARDUINOPORT "/dev/ttyUSB0"
#define ARDUINOPORT "/dev/ttyACM0"

#define STEP_X                6
#define DIR_X                 5
#define M3_X                 26

#define STEP_Y                4
#define DIR_Y                 1

#define STEP_CUTTER          16
#define DIR_CUTTER           15

#define STEPPER_EN           29

#define COM_1                27
#define COM_2                28

#define SWITCH_CUTTER         3
#define SWITCH_PRINTHEAD      2

#define SWITCH_LCD_L         13
#define SWITCH_LCD_M         12
#define SWITCH_LCD_R         14


#define STEP_COMMAND_PAUSE    5
#define STEP_PAUSE          100

#define DOT_WIDTH            11
#define DOT_HEIGHT           42

#define CUT_LENGTH        42000
#define STARTPUNKT       -33900
#define PI 3.1415927

#define LCD_RS     24
#define LCD_EN     25
#define LCD_D0     30
#define LCD_D1     21
#define LCD_D2     22
#define LCD_D3     23

#define LCD_ROWS    2
#define LCD_COLS   16
#define LCD_BITS    4


char FileName[1000] = "";

//Global lcd handle:
static int lcdHandle ;

int fd = 0;
struct termios TermOpt;

int plotterSteps = 0;

char PicturePath[1000];

long posX = 0;
long posY = 0;
double cutCount = 0.0;

int readCount;
char nozzles[12];

time_t t;
struct tm tm;
double WorldPos = -180.0;
long WorldShouldTime;
long WorldIsTime;

char LCD_line_1[20];
char LCD_line_2[20];


//+++++++++++++++++++++++ LCD display while printing +++++++++++++++++++
void printingLCD(){
  lcdClear (lcdHandle);
  t = time(NULL);
  tm = *localtime(&t);
  printf("now: %d-%d-%d %d:%d:%d\n", tm.tm_year + 1900, tm.tm_mon + 1, tm.tm_mday, tm.tm_hour, tm.tm_min, tm.tm_sec);
  sprintf(LCD_line_1, "Uhrzeit: %02d:%02d", tm.tm_hour, tm.tm_min);
  if(WorldPos < 0.0){
    sprintf(LCD_line_2, "Grad Ost  %6.2lf", -WorldPos);
  }
  else{
    sprintf(LCD_line_2, "Grad West %6.2lf", WorldPos);
  }
  if(WorldPos >= 179.9){
    sprintf(LCD_line_1, "Uhrzeit: 00:00");
  }
  if(WorldPos >= 0.0 && WorldPos < 0.2){
    sprintf(LCD_line_1, "Uhrzeit: 12:00");
  }
  if(WorldPos > 180.0){
    sprintf(LCD_line_2, "Grad Ost  %6.2lf", -(WorldPos-360.0));
  }
  lcdPosition(lcdHandle, 0, 0);
  lcdPuts (lcdHandle, LCD_line_1);
  lcdPosition(lcdHandle, 0, 1);
  lcdPuts (lcdHandle, LCD_line_2);
}
//----------------------- LCD display while printing -------------------


//+++++++++++++++++++++++ Send nozzle data to Arduino ++++++++++++++++++
void send2arduino(){

/*
  printf("Sending ");
  for(int i = 0; i < 12; i++){
    printf("%d, ", nozzles[i]);
  }
  printf("... ");
*/
  for(int i = 0; i < 12; i++){
    if(nozzles[i] == 1){
      digitalWrite(COM_2, 1);
    }
    else{
      digitalWrite(COM_2, 0);
    }
    digitalWrite(COM_1, 1);
    usleep(5);
    digitalWrite(COM_1, 0);
    usleep(5);
  }
  digitalWrite(COM_2, 0);
  usleep(500);
//  printf(" ...done!\n");
}
//----------------------- Send nozzle data to Arduino ------------------

//+++++++++++++++++++++++ Start readport +++++++++++++++++++++++++++++++
char  readport(void){
  int n;
  char buff;
  n = read(fd, &buff, 1);
  if(n > 0){
    return buff;
  }
  return 0;
}
//------------------------ End readport --------------------------------

//+++++++++++++++++++++++ Start sendport +++++++++++++++++++++++++++++++
void sendport(unsigned char ValueToSend){
  int n;

  n = write(fd, &ValueToSend, 1);

  if (n < 0){
    printf("write() of value failed!\r");
  }
  else{
    if(readCount == 1){
      while(readport() != 'R');
      readCount = 0;
    }
    readCount++;
  }

}
//------------------------ End sendport --------------------------------

//+++++++++++++++++++++++ Start openport +++++++++++++++++++++++++++++++
void openport(void){

    fd = open(ARDUINOPORT, O_RDWR | O_NOCTTY | O_NDELAY);
    if (fd == -1)  {
      printf("init_serialport: Unable to open port ");
    }

    if (tcgetattr(fd, &TermOpt) < 0) {
      printf("init_serialport: Couldn't get term attributes");
    }
    speed_t brate = BAUDRATE; // let you override switch below if needed

    cfsetispeed(&TermOpt, brate);
    cfsetospeed(&TermOpt, brate);

    // 8N1
    TermOpt.c_cflag &= ~PARENB;
    TermOpt.c_cflag &= ~CSTOPB;
    TermOpt.c_cflag &= ~CSIZE;
    TermOpt.c_cflag |= CS8;
    // no flow control
    TermOpt.c_cflag &= ~CRTSCTS;

    TermOpt.c_cflag |= CREAD | CLOCAL;  // turn on READ & ignore ctrl lines
    TermOpt.c_iflag &= ~(IXON | IXOFF | IXANY); // turn off s/w flow ctrl

    TermOpt.c_lflag &= ~(ICANON | ECHO | ECHOE | ISIG); // make raw
    TermOpt.c_oflag &= ~OPOST; // make raw

    // see: http://unixwiz.net/techtips/termios-vmin-vtime.html
    TermOpt.c_cc[VMIN]  = 0;
    TermOpt.c_cc[VTIME] = 20;

    if( tcsetattr(fd, TCSANOW, &TermOpt) < 0) {
      printf("init_serialport: Couldn't set term attributes");
    }

}
//------------------------ End openport --------------------------------


//++++++++ Make ONE step X motor  and send ONe Nozle to Arduino ++++++++
void makeStepXandNozzle(long direction, long stepPause, int nozzleData){

  if(direction == 1){
    digitalWrite(DIR_X, 1);
    posX++;
  }
  if(direction == -1){
    digitalWrite(DIR_X, 0);
    posX--;
  }

  if(direction == 1 || direction == -1){
    digitalWrite(STEP_X, HIGH);
    usleep(STEP_COMMAND_PAUSE);
    digitalWrite(STEP_X, LOW);
  }
  if(nozzleData > -1 && nozzleData < 12){
    if(nozzles[nozzleData] == 1){
      digitalWrite(COM_2, 1);
    }
    else{
      digitalWrite(COM_2, 0);
    }
    digitalWrite(COM_1, 1);
    usleep(5);
    digitalWrite(COM_1, 0);
    usleep(5);
    digitalWrite(COM_2, 0);
  }
  if(direction == 1 || direction == -1){
    usleep(stepPause);
  }


}
//-------- Make ONE step X motor  and send ONe Nozle to Arduino --------

//+++++++++++++++++++++++ Make step X motor ++++++++++++++++++++++++++++
void makeStepX(long NewPosX, long stepPause){
  if(NewPosX - posX > 0){
    digitalWrite(DIR_X, 1);
    while(posX < NewPosX){
      digitalWrite(STEP_X, HIGH);
      usleep(STEP_COMMAND_PAUSE);
      digitalWrite(STEP_X, LOW);
      usleep(stepPause);
      posX++;
    }
  }
  else{
    digitalWrite(DIR_X, 0);
    while(posX > NewPosX){
      digitalWrite(STEP_X, HIGH);
      usleep(STEP_COMMAND_PAUSE);
      digitalWrite(STEP_X, LOW);
      usleep(stepPause);
      posX--;
    }
  }
}
//------------------------ Make step X motor ---------------------------

//+++++++++++++++++++++++ Make step Y motor ++++++++++++++++++++++++++++
void makeStepY(long stepsY, long stepPause){
  posY += stepsY;
  if(stepsY > 0){
    digitalWrite(DIR_Y, 0);
  }
  else{
    digitalWrite(DIR_Y, 1);
    stepsY = -stepsY;
  }
  for(long i = 0; i < stepsY; i++){
    digitalWrite(STEP_Y, HIGH);
    usleep(STEP_COMMAND_PAUSE);
    digitalWrite(STEP_Y, LOW);
    usleep(stepPause);
  }

}
//------------------------ Make step Y motor ---------------------------

//+++++++++++++++++++++++ Make step Cutter motor +++++++++++++++++++++++
void makeStepCutter(long stepsCut, long stepPause){
  if(stepsCut > 0){
    digitalWrite(DIR_CUTTER, 0);
  }
  else{
    digitalWrite(DIR_CUTTER, 1);
    stepsCut = -stepsCut;
  }
  for(long i = 0; i < stepsCut; i++){
    digitalWrite(STEP_CUTTER, HIGH);
    usleep(STEP_COMMAND_PAUSE);
    digitalWrite(STEP_CUTTER, LOW);
    usleep(stepPause);
  }

}
//------------------------ Make step Cutter motor ----------------------

//+++++++++++++++++++++++ Pause print ++++++++++++++++++++++++++++++++++
void pausePrint(){
  int resumePrint = 0;
  long posXold = posX;

  lcdPosition(lcdHandle, 0, 0);
  lcdPuts (lcdHandle, "Druck pausiert: ");
  lcdPosition(lcdHandle, 0, 1);
  lcdPuts (lcdHandle, "L=Parken R=Forts");
  while(digitalRead(SWITCH_LCD_M) == 0);
  sleep(1);
  while(resumePrint == 0){

      if(digitalRead(SWITCH_LCD_R) == 0){
        lcdPosition(lcdHandle, 0, 0);
        lcdPuts (lcdHandle, "Setze Druck     ");
        lcdPosition(lcdHandle, 0, 1);
        lcdPuts (lcdHandle, "fort...         ");
        sleep(1);
        while(digitalRead(SWITCH_LCD_R) == 0);
        sleep(1);
        resumePrint = 1;
      }

      if(digitalRead(SWITCH_LCD_L) == 0){
        while(digitalRead(SWITCH_CUTTER) == 1){
         makeStepCutter(-10, STEP_PAUSE/4);
        }
        printf("Endschalter Cutter erreicht!\n");
        while(digitalRead(SWITCH_CUTTER) == 0){
          makeStepCutter(10, STEP_PAUSE/4);
        }
        printf("Endschalter Cutter wieder frei!\n");

        while(digitalRead(SWITCH_PRINTHEAD) == 1){
          posX = 0;
          makeStepX(10, STEP_PAUSE/4);
        }
        printf("Endschalter Druckkopf erreicht!\n");
        while(digitalRead(SWITCH_PRINTHEAD) == 0){
        posX = 0;
        makeStepX(-10, STEP_PAUSE/4);
        }
        posX = 0;
        printf("Endschalter Druckkopf wieder frei!\n");
        lcdPosition(lcdHandle, 0, 0);
        lcdPuts (lcdHandle, "Beliebige Taste ");
        lcdPosition(lcdHandle, 0, 1);
        lcdPuts (lcdHandle, "wenn fertig.    ");
        while(digitalRead(SWITCH_LCD_L) == 1 && digitalRead(SWITCH_LCD_M) == 1 &&digitalRead(SWITCH_LCD_R) == 1);

        lcdPosition(lcdHandle, 0, 0);
        lcdPuts (lcdHandle, "Fahre zur Aus-  ");
        lcdPosition(lcdHandle, 0, 1);
        lcdPuts (lcdHandle, "gangsposition...");

        makeStepX(STARTPUNKT, STEP_PAUSE/4);
        posX = 0;

        makeStepX(posXold, STEP_PAUSE/4);

        lcdPosition(lcdHandle, 0, 0);
        lcdPuts (lcdHandle, "Druck pausiert: ");
        lcdPosition(lcdHandle, 0, 1);
        lcdPuts (lcdHandle, "L=Papier R=Forts");

      }
  }
}
//----------------------- Pause print ----------------------------------



//######################################################################
//################## Main ##############################################
//######################################################################

int main(int argc, char **argv){

  char FullFileName[1000] = "";
  char *pEnd;
  FILE *bmpFile;
  char TextLine[30000];
  char a;

  char FileInfo[3];
  long FileSize;
  long LongTemp;
  long DataOffset;
  long HeaderSize;
  long PictureWidth;
  long PictureHeight;
  int  IntTemp;
  int  ColorDepth;
  long CompressionType;
  long PictureSize;
  long XPixelPerMeter;
  long YPixelPerMeter;
  long ColorNumber;
  long ColorUsed;
  int PixelRed, PixelGreen, PixelBlue;
  int PixOK=0;
  long BrightestX=0, BrightestY=0;
  int BrightR = 0, BrightG = 0, BrightB = 0;
  int ServoX = 150, ServoY = 150;
  long CurrentPicNo = 1, LastPicNo = 1;


  int FirstPrint = 1;

  # define BITMAP_MAX  100000
  unsigned char bitmapData[BITMAP_MAX];

  unsigned char dataToSend1, dataToSend2;

  long PictureWidthFilled = 0;


  readCount = 0;


  if (wiringPiSetup () == -1){
    printf("Could not run wiringPiSetup!");
    exit(1);
  }

  lcdHandle = lcdInit(LCD_ROWS, LCD_COLS, LCD_BITS, LCD_RS, LCD_EN, LCD_D0, LCD_D1, LCD_D2, LCD_D3,0,0,0,0);

  if(lcdHandle < 0){
    fprintf(stderr, "%s: lcdInit failed\n", argv [0]);
    return -1;
  }

  lcdClear (lcdHandle);

  pinMode (STEP_X, OUTPUT);
  pinMode (DIR_X, OUTPUT);
  pinMode (M3_X, OUTPUT);
  digitalWrite(M3_X, 1);

  pinMode (STEP_Y, OUTPUT);
  pinMode (DIR_Y, OUTPUT);

  pinMode (STEP_CUTTER, OUTPUT);
  pinMode (DIR_CUTTER, OUTPUT);

  pinMode (STEPPER_EN, OUTPUT);
  digitalWrite(STEPPER_EN, 0); // Turn ON stepper motors (pin is NOT ENABLE)

  pinMode (COM_1, OUTPUT);
  pinMode (COM_2, OUTPUT);

  pinMode (SWITCH_CUTTER, INPUT);
  pullUpDnControl(SWITCH_CUTTER, PUD_OFF);

  pinMode (SWITCH_PRINTHEAD, INPUT);
  pullUpDnControl(SWITCH_PRINTHEAD, PUD_OFF);

  pinMode (SWITCH_LCD_L, INPUT);
  pullUpDnControl(SWITCH_LCD_L, PUD_UP);

  pinMode (SWITCH_LCD_M, INPUT);
  pullUpDnControl(SWITCH_LCD_M, PUD_UP);

  pinMode (SWITCH_LCD_R, INPUT);
  pullUpDnControl(SWITCH_LCD_R, PUD_UP);


  char dateCommand[100];


  while(digitalRead(SWITCH_LCD_M) == 1){
    t = time(NULL);
    tm = *localtime(&t);

    if(digitalRead(SWITCH_LCD_L) == 0){
      printf("now: %d-%d-%d %d:%d:%d\n", tm.tm_year + 1900, tm.tm_mon + 1, tm.tm_mday, tm.tm_hour, tm.tm_min, tm.tm_sec);
      if(tm.tm_min > 0){
        tm.tm_min--;
      }
      else{
        tm.tm_min = 59;
        if(tm.tm_hour > 0){
          tm.tm_hour--;
        }
        else{
          tm.tm_hour = 23;
        }
      }
      sprintf(dateCommand, "sudo date -s '%04d%02d%02d %02d:%02d:00'", tm.tm_year + 1900, tm.tm_mon + 1, tm.tm_mday,tm.tm_hour,tm.tm_min);
      printf("%s\n", dateCommand);
      system(dateCommand);
      //printf("now: %d-%d-%d %d:%d:%d\n", tm.tm_year + 1900, tm.tm_mon + 1, tm.tm_mday, tm.tm_hour, tm.tm_min, tm.tm_sec);
      sprintf(LCD_line_1, "Uhrzeit: %02d:%02d", tm.tm_hour, tm.tm_min);
      lcdPosition(lcdHandle, 0, 0);
      lcdPuts (lcdHandle, LCD_line_1);
      lcdPosition(lcdHandle, 0, 1);
      lcdPuts (lcdHandle, "L-    M=OK    R+");
      usleep(50000);
    }

    if(digitalRead(SWITCH_LCD_R) == 0){
      printf("now: %d-%d-%d %d:%d:%d\n", tm.tm_year + 1900, tm.tm_mon + 1, tm.tm_mday, tm.tm_hour, tm.tm_min, tm.tm_sec);
      if(tm.tm_min < 59){
        tm.tm_min++;
      }
      else{
        tm.tm_min = 0;
        if(tm.tm_hour < 23){
          tm.tm_hour++;
        }
        else{
          tm.tm_hour = 0;
        }
      }
      sprintf(dateCommand, "sudo date -s '%04d%02d%02d %02d:%02d:00'", tm.tm_year + 1900, tm.tm_mon + 1, tm.tm_mday,tm.tm_hour,tm.tm_min);
      printf("%s\n", dateCommand);
      system(dateCommand);
      //printf("now: %d-%d-%d %d:%d:%d\n", tm.tm_year + 1900, tm.tm_mon + 1, tm.tm_mday, tm.tm_hour, tm.tm_min, tm.tm_sec);
      sprintf(LCD_line_1, "Uhrzeit: %02d:%02d", tm.tm_hour, tm.tm_min);
      lcdPosition(lcdHandle, 0, 0);
      lcdPuts (lcdHandle, LCD_line_1);
      lcdPosition(lcdHandle, 0, 1);
      lcdPuts (lcdHandle, "L-    M=OK    R+");
      usleep(50000);
    }


    //printf("now: %d-%d-%d %d:%d:%d\n", tm.tm_year + 1900, tm.tm_mon + 1, tm.tm_mday, tm.tm_hour, tm.tm_min, tm.tm_sec);
    sprintf(LCD_line_1, "Uhrzeit: %02d:%02d", tm.tm_hour, tm.tm_min);
    lcdPosition(lcdHandle, 0, 0);
    lcdPuts (lcdHandle, LCD_line_1);
    lcdPosition(lcdHandle, 0, 1);
    lcdPuts (lcdHandle, "L-    M=OK    R+");


  }

  system("sudo hwclock -w");

//  exit(0);//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!



  lcdPosition(lcdHandle, 0, 0);
  lcdPuts (lcdHandle, "  World Printer ") ;
  lcdPosition(lcdHandle, 0, 1);
  lcdPuts (lcdHandle, "...initialisiere") ;
  sleep(2);


  //exit(0);//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

//  openport();

//  printf("\n\nWaiting for 'X' from Arduino (Arduino pluged in?)...\n");

  //Wait for 'X' from Arduino
//  while(readport() != 'X');
//  printf("Connection to Arduino established.\n");

/*
  // Test timeout Nozzle data
  send2arduino();
  digitalWrite(COM_1, 1);
  usleep(3000000);
  send2arduino();
  digitalWrite(COM_1, 1);
  usleep(3000000);
  send2arduino();
  digitalWrite(COM_1, 1);
  usleep(3000000);
  exit(0);//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
*/


/*
// Test cutter

//  sleep(10);


  makeStepY(DOT_HEIGHT*10000, STEP_PAUSE*4);
  exit(0);//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

  sleep(1);

  makeStepCutter(CUT_LENGTH, STEP_PAUSE/4);
  while(digitalRead(SWITCH_CUTTER) == 1){
    makeStepCutter(-10, STEP_PAUSE/4);
  }
  printf("Endschalter Cutter erreicht!\n");
  while(digitalRead(SWITCH_CUTTER) == 0){
    makeStepCutter(10, STEP_PAUSE/4);
  }
  printf("Endschalter Cutter wieder frei!\n");
  exit(0);//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
*/

/*
  // Test nozzles
  for(int i = 0; i < 12; i++){
    for(int k = 0; k < 2; k++){
      if(k==0){
        makeStepX(500, STEP_PAUSE);
        makeStepX(-500, STEP_PAUSE);
      }
      else{
        makeStepX(-500, STEP_PAUSE);
        makeStepX(500, STEP_PAUSE);
      }
      for(int j = 0; j < 100; j++){
        for(int l = 0; l < 12; l++){
          nozzles[l] = 0;
        }
        nozzles[i] = 1;
        send2arduino();
        makeStepY(DOT_WIDTH, STEP_PAUSE);
        usleep(1000);
      }
    }
  }
  exit(0);// !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
*/

/*
  // Test Switches
  printf("Press Cutter switch...\n");
  while(digitalRead(SWITCH_CUTTER) == 1);
  printf("...done!\n");

  printf("Press Printhead switch...\n");
  while(digitalRead(SWITCH_PRINTHEAD) == 1);
  printf("...done!\n");

  printf("Press Left switch on LCD...\n");
  while(digitalRead(SWITCH_LCD_L) == 1);
  printf("...done!\n");

  printf("Press Center switch on LCD...\n");
  while(digitalRead(SWITCH_LCD_M) == 1);
  printf("...done!\n");

  printf("Press Right switch on LCD...\n");
  while(digitalRead(SWITCH_LCD_R) == 1);
  printf("...done!\n");

  exit(0);// !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
*/

  while(digitalRead(SWITCH_CUTTER) == 1){
    makeStepCutter(-10, STEP_PAUSE/4);
  }
  printf("Endschalter Cutter erreicht!\n");
  while(digitalRead(SWITCH_CUTTER) == 0){
    makeStepCutter(10, STEP_PAUSE/4);
  }
  printf("Endschalter Cutter wieder frei!\n");

  while(digitalRead(SWITCH_PRINTHEAD) == 1){
    posX = 0;
    makeStepX(10, STEP_PAUSE/4);
  }
  printf("Endschalter Druckkopf erreicht!\n");
  while(digitalRead(SWITCH_PRINTHEAD) == 0){
    posX = 0;
    makeStepX(-10, STEP_PAUSE/4);
  }
  posX = 0;
  printf("Endschalter Druckkopf wieder frei!\n");

  lcdPosition(lcdHandle, 0, 0);
  lcdPuts (lcdHandle, "Mittlere Taste  ") ;
  lcdPosition(lcdHandle, 0, 1);
  lcdPuts (lcdHandle, "startet Druck...") ;

  printf("Mittleren Schalter am LCD drücken...\n");
  while(digitalRead(SWITCH_LCD_M) == 1);
  printf("...starte Druck!\n");

  lcdPosition(lcdHandle, 0, 0);
  lcdPuts (lcdHandle, "Ich starte den  ") ;
  lcdPosition(lcdHandle, 0, 1);
  lcdPuts (lcdHandle, "Weltendruck...  ") ;


  // Druckkopf zum Startpunkt fahren
  makeStepX(STARTPUNKT, STEP_PAUSE/4);
  posX = 0;

  printf("Startposition Druckkopf erreicht!\n");

  printingLCD();

//  exit(0);// !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!


/*
  makeStepX(DOT_WIDTH*20, STEP_PAUSE);
  makeStepX(-DOT_WIDTH*20, STEP_PAUSE);

  makeStepY(DOT_WIDTH*20, STEP_PAUSE);
  makeStepY(-DOT_WIDTH*20, STEP_PAUSE);
*/


//  strcpy(FileName, "tux-caveman.bmp");
//  strcpy(FileName, "square.bmp");
//  strcpy(FileName, "square-thin.bmp");
  strcpy(FileName, "WorldMap.bmp");

  getcwd(PicturePath, 1000);
  strcat(PicturePath, "/pictures");
  printf("PicturePath=>%s<", PicturePath);


  while(1==1){
      if((bmpFile=fopen(FileName,"rb"))==NULL){
        printf("Datei '%s' kann nicht zum Lesen geoeffnet werden!\n", FileName);
        return(1);
      }

      fread(&FileInfo, 2, 1, bmpFile);
      fread(&FileSize, 4, 1, bmpFile);
      fread(&LongTemp, 4, 1, bmpFile);
      fread(&DataOffset, 4, 1, bmpFile);
      fread(&HeaderSize, 4, 1, bmpFile);
      fread(&PictureWidth, 4, 1, bmpFile);
      fread(&PictureHeight, 4, 1, bmpFile);
      fread(&IntTemp, 2, 1, bmpFile);
      fread(&ColorDepth, 2, 1, bmpFile);
      fread(&CompressionType, 4, 1, bmpFile);
      fread(&PictureSize, 4, 1, bmpFile);
      fread(&XPixelPerMeter, 4, 1, bmpFile);
      fread(&YPixelPerMeter, 4, 1, bmpFile);
      fread(&ColorNumber, 4, 1, bmpFile);
      fread(&ColorUsed, 4, 1, bmpFile);

      printf("FileInfo=%s\n", FileInfo);
      printf("FileSize=%ld\n", FileSize);
      printf("LongTemp=%ld\n", LongTemp);
      printf("DataOffset=%ld\n", DataOffset);
      printf("HeaderSize=%ld\n", HeaderSize);
      printf("PictureWidth=%ld\n", PictureWidth);
      printf("PictureHeight=%ld\n", PictureHeight);
      printf("IntTemp=%d\n", IntTemp);
      printf("ColorDepth=%d\n", ColorDepth);
      printf("CompressionType=%ld\n", CompressionType);
      printf("PictureSize=%ld\n", PictureSize);
      printf("XPixelPerMeter=%ld\n", XPixelPerMeter);
      printf("YPixelPerMeter=%ld\n", YPixelPerMeter);
      printf("ColorNumber=%ld\n", ColorNumber);
      printf("ColorUsed=%ld\n", ColorUsed);


      if(FileInfo[0] != 'B' || FileInfo[1] != 'M'){
        printf("Wrong Fileinfo (BM)!\n");
        return(1);
      }
      if(ColorDepth != 24){
        printf("Wrong ColorDepth (24)!\n");
        return(1);
      }
      if(CompressionType != 0){
        printf("Wrong CompressionType (0)!\n");
        return(1);
      }


      fseek(bmpFile, DataOffset, SEEK_SET);

      PictureWidthFilled = PictureWidth*3;
      while(PictureWidthFilled%4 != 0){
        PictureWidthFilled++;
      }

      printf("PictureWidthFilled = %ld, PictureWidth = %ld\n", PictureWidthFilled, PictureWidth);


      int InkPerDot = 3;
      char dot2nozzles[PictureWidth][12];
      long X;
      //long offsetX;

      WorldPos = -180;
      WorldPos += 360.0 / (double)(PictureHeight);// Little shift to make sure Meridians are printed correctly

      while(!feof(bmpFile)){
        for(int reverse = 0; reverse < 2; reverse++){

          //printf("reverse=%d\n", reverse);
           // Reset buffer to 255 (White)
          for(int j = 0; j < BITMAP_MAX; j++){
            bitmapData[j]=255;
          }
          if(FirstPrint == 1){
            long SkipLines = 0;
            FirstPrint = 0;
            t = time(NULL);
            tm = *localtime(&t);

            SkipLines = (long)((double)(PictureHeight) * (tm.tm_min + 60 * tm.tm_hour) / (24 * 60));
            if(SkipLines > 0){
              for(long i = 0; i < SkipLines; i++){
                if(!feof(bmpFile)){
                  fread(bitmapData, 1, PictureWidthFilled, bmpFile);
                  WorldPos += 360.0 / (double)(PictureHeight);
                }
              }
            }

          }
          // Read line into buffer
          if(!feof(bmpFile)){
            fread(bitmapData, 1, PictureWidthFilled, bmpFile);
          }
          unsigned char BWline[PictureWidth+1];
          for(int i = 0; i < PictureWidth; i++){
            if(bitmapData[i*3]<127 && bitmapData[i*3+1]<127 && bitmapData[i*3+2]<127){// Black dot
               BWline[i] = 'X';
            }
            else{
              BWline[i] = ' ';
            }
            BWline[i+1] = '\0';
          }


          if(reverse == 0){
            X = -24;
            //offsetX = 0;
            makeStepX(X * DOT_WIDTH, STEP_PAUSE);
          }
          else{
            X = PictureWidth + 24;
            //offsetX = DOT_WIDTH * 12;
            makeStepX(X * DOT_WIDTH, STEP_PAUSE);
            //unsigned char BWtemp;
            //for(int i = 0; i < PictureWidth / 2; i++){
            //  BWtemp = BWline[i];
            //  BWline[i] = BWline[PictureWidth - 1 - i];
            //  BWline[PictureWidth - 1 - i] = BWtemp;
            //}
          }


          //printf("%s\n", BWline);
          srand(time(NULL));
          // random int = (rand() % (max + 1 - min)) + min
          int nozzleCountStart = rand() % (12);
          int nozzleCount;
          for(int j = 0; j < InkPerDot; j++){
            nozzleCountStart += 5;
            if(nozzleCountStart > 11)nozzleCountStart -= 12;
            nozzleCount = nozzleCountStart;
            for(long i = 0; i < PictureWidth; i++){
              dot2nozzles[i][j] = nozzleCount;
              nozzleCount++;
              if(nozzleCount > 11)nozzleCount = 0;
            }
          }


          while(X >= -24 && X <= PictureWidth + 24){
            for(int j = 0; j < 24; j += 2){
              nozzles[j / 2] = 0;
              if(X + j < PictureWidth && X + j > -1){
                if(BWline[X + j] == 'X'){// Black dot
                  for(int k = 0; k < InkPerDot; k++){
                    if(dot2nozzles[X + j][k] == j / 2){
                      nozzles[j / 2] = 1;
                    }
                  }
                }
              }
            }
            //printf(" dataToSend1=%d, dataToSend2=%d\n", dataToSend1, dataToSend2);
            for(int i = 0; i < DOT_WIDTH; i++){
              makeStepXandNozzle((1-2*reverse), STEP_PAUSE / 2, i);
            }
            if(DOT_WIDTH < 12){
              for(int i = DOT_WIDTH; i < 12; i++){
                makeStepXandNozzle(0, STEP_PAUSE / 2, i);
              }
            }
            X += (1-2*reverse);
            if(digitalRead(SWITCH_LCD_M) == 0){
              pausePrint();
            }
          }
          if(!feof(bmpFile)){
            makeStepY(DOT_HEIGHT, STEP_PAUSE);
          }

          for(int i = 0; i < 100;i++){
            usleep(10000L);
            if(digitalRead(SWITCH_LCD_M) == 0){
              pausePrint();
            }
          }

          WorldShouldTime = (long)((24.0 * 60.0 * 60.0) * (WorldPos + 180) / 360.0);
          int timeOK = 0;

          while(timeOK == 0){
            t = time(NULL);
            tm = *localtime(&t);
            WorldIsTime = tm.tm_hour * 60 * 60 + tm.tm_min * 60 + tm.tm_sec;
            if(WorldIsTime >= WorldShouldTime){
              timeOK = 1;
            }
            if(WorldShouldTime - WorldIsTime > 120){// Don't wait longer than 2 minutes
              timeOK = 1;
            }
            if(WorldIsTime < 1){
              timeOK = 1;
            }
            if(digitalRead(SWITCH_LCD_M) == 0){
              pausePrint();
            }
          }



          cutCount++;
          WorldPos += 360.0 / (double)(PictureHeight);

          printingLCD();
          t = time(NULL);
          tm = *localtime(&t);
          if(cutCount > 4*40.885416667 || ((tm.tm_min == 0 || tm.tm_min == 30) && cutCount > 10) ){
            cutCount = 0;
            makeStepCutter(CUT_LENGTH, STEP_PAUSE/4);
            while(digitalRead(SWITCH_CUTTER) == 1){
              makeStepCutter(-10, STEP_PAUSE/4);
            }
            printf("Endschalter Cutter erreicht!\n");
            while(digitalRead(SWITCH_CUTTER) == 0){
              makeStepCutter(10, STEP_PAUSE/4);
            }
            printf("Endschalter Cutter wieder frei!\n");

          }
          printf("cutCount=%lf\n", cutCount);
        }
      }

      makeStepX(0, STEP_PAUSE);


      fclose(bmpFile);
  }
  digitalWrite(STEPPER_EN, 1); // Turn off stepper motors (pin is NOT ENABLE)

  return 0;
}
